package com.muyu.marketing.service.impl;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.utils.StringUtils;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.marketing.domain.ActivityTeamInfo;
import com.muyu.marketing.domain.model.ActivityTeamInfoListModel;
import com.muyu.marketing.domain.model.ActivityTeamInfoListQueryModel;
import com.muyu.marketing.domain.model.TeamProductDiscountPriceModel;
import com.muyu.marketing.domain.model.TeamProductStockModel;
import com.muyu.marketing.mapper.ActivityInfoMapper;
import com.muyu.marketing.mapper.ActivityTeamInfoMapper;
import com.muyu.marketing.service.ActivityOpenInfoService;
import com.muyu.marketing.service.ActivityTeamInfoService;
import com.muyu.marketing.service.ActivityTeamProductSkuInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Stream;

@Service
public class ActivityTeamInfoServiceImpl extends ServiceImpl<ActivityTeamInfoMapper,ActivityTeamInfo> implements ActivityTeamInfoService {

    @Autowired
    private ActivityTeamProductSkuInfoService activityTeamProductSkuInfoService;

    @Autowired
    private ActivityOpenInfoService activityOpenInfoService;

    @Override
    public TableDataInfo<ActivityTeamInfoListModel> query (ActivityTeamInfoListQueryModel activityTeamInfoListQueryModel) {
        LambdaQueryWrapper<ActivityTeamInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.like(StringUtils.isNotEmpty(activityTeamInfoListQueryModel.getKeyWord()),ActivityTeamInfo::getName , activityTeamInfoListQueryModel.getKeyWord());
        queryWrapper.like(StringUtils.isNotEmpty(activityTeamInfoListQueryModel.getStatus()),ActivityTeamInfo::getStatus , activityTeamInfoListQueryModel.getStatus());
        Page<ActivityTeamInfo>  activityTeamInfoPage = this.page(activityTeamInfoListQueryModel.buildPage(),queryWrapper);
        List<ActivityTeamInfo> activityTeamInfoList = activityTeamInfoPage.getRecords();
        List<ActivityTeamInfoListModel> activityTeamInfoListModels = activityTeamInfoList.stream()
                .map(activityTeamInfo -> ActivityTeamInfoListModel.infoBuild(activityTeamInfo,
                                (activityTeamInfoListModelBuilder) -> {
                          Long openNumberByTeamIdAndType = activityOpenInfoService.getTeamOpenNumberByTeamIdAndType(activityTeamInfo.getId());
                          Long inNumberByTeamIdAndType = activityOpenInfoService.getTeamInNumberByTeamIdAndType(activityTeamInfo.getId());
                          TeamProductDiscountPriceModel discountPrice = activityTeamProductSkuInfoService.getDiscountPrice(activityTeamInfo.getId());
                          TeamProductStockModel teamProductStockModel = activityTeamProductSkuInfoService.getStock(activityTeamInfo.getId());

                                return  activityTeamInfoListModelBuilder
                                        .openTeamNumber(openNumberByTeamIdAndType)
                                        .addTeamNumber(inNumberByTeamIdAndType)
                                        .attendNumber(openNumberByTeamIdAndType + inNumberByTeamIdAndType)
                                        .teamPrice(discountPrice.getTeamPrice())
                                        .productPrice(discountPrice.getProductPrice())
                                        .teamStock(teamProductStockModel.getTeamStock())
                                        .remainStock(teamProductStockModel.getRemainStock())
                                         .build();

                        })).toList();
        TableDataInfo<ActivityTeamInfoListModel> tableDataInfo = new TableDataInfo<>();
        tableDataInfo.setTotal(activityTeamInfoPage.getTotal());
        tableDataInfo.setRows(activityTeamInfoListModels);

        return tableDataInfo;

    }
}
