package com.muyu.marketing.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.marketing.domain.ActivityTeamProductSkuInfo;
import com.muyu.marketing.domain.model.TeamProductDiscountPriceModel;
import com.muyu.marketing.domain.model.TeamProductStockModel;
import com.muyu.marketing.mapper.ActivityTeamOpenInfoMapper;
import com.muyu.marketing.mapper.ActivityTeamProductSkuInfoMapper;
import com.muyu.marketing.service.ActivityTeamProductSkuInfoService;
import com.muyu.product.cache.ProjectSkuCache;
import com.muyu.product.cache.datasource.ProjectSkuData;
import com.muyu.product.domain.ProjectSkuInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.List;
@Service
public class ActivityTeamProductSkuInfoServiceImpl extends ServiceImpl<ActivityTeamProductSkuInfoMapper,ActivityTeamProductSkuInfo> implements ActivityTeamProductSkuInfoService {

    @Autowired
    private ProjectSkuCache projectSkuCache;

    @Override
    public TeamProductDiscountPriceModel getDiscountPrice(Long teamId) {
        LambdaQueryWrapper<ActivityTeamProductSkuInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(ActivityTeamProductSkuInfo::getTeamId,teamId);
        List<ActivityTeamProductSkuInfo> productSkuInfos = this.list(queryWrapper);
        ArrayList<TeamProductDiscountPriceModel> teamProductDiscountPriceModels = new ArrayList<>();
        for (ActivityTeamProductSkuInfo productSkuInfo : productSkuInfos) {
            ProjectSkuInfo skuInfo = projectSkuCache.get(productSkuInfo.getProductId(), productSkuInfo.getProductSku());
            BigDecimal price = skuInfo.getPrice();
            BigDecimal teamPrice = productSkuInfo.getTeamPrice();
            TeamProductDiscountPriceModel priceModel = new TeamProductDiscountPriceModel();
            priceModel.setProductPrice(price);
            priceModel.setTeamPrice(teamPrice);
            priceModel.setDisCount(price.subtract(teamPrice).divide(price,2, RoundingMode.HALF_UP).doubleValue());

            teamProductDiscountPriceModels.add(priceModel);
        }
        TeamProductDiscountPriceModel teamProductDiscountPriceModel = teamProductDiscountPriceModels.get(0);
        for (TeamProductDiscountPriceModel productDiscountPriceModel : teamProductDiscountPriceModels) {
            if(teamProductDiscountPriceModel.getDisCount() < productDiscountPriceModel.getDisCount()){
                teamProductDiscountPriceModel = productDiscountPriceModel;
            }
        }
        return teamProductDiscountPriceModel;

    }

    @Override
    public TeamProductStockModel getStock(Long teamId) {
        List<ActivityTeamProductSkuInfo> teamProductSkuInfosList = this.getActivityTeamProductSkuInfoByTeamId(teamId);

        return TeamProductStockModel.builder()
                .teamStock(teamProductSkuInfosList.stream().map(ActivityTeamProductSkuInfo::getTeamStock).reduce(0L,Long::sum))
                .remainStock(teamProductSkuInfosList.stream().map(ActivityTeamProductSkuInfo::getRemainStock).reduce(0L,Long::sum))
                .build();
    }
}
